/*
 * Title:        sag_internal/epl_plugin_functions.hpp
 * Description:  C function ABI for EPL Plugins
 * $Copyright (c) 2015, 2016, 2022 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 */

#ifndef _SAG_INTERNAL_C_EPLPLUGIN_FUNCTIONS_HPP_
#define _SAG_INTERNAL_C_EPLPLUGIN_FUNCTIONS_HPP_

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/** Sends an event to a context */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_send_data_ctx(void *, const char *, sag_underlying_data_t, uint64_t);

/** Sends an event to a channel */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_send_data(void *, const char *, sag_underlying_data_t, const char *);

/** Checks the correlator version against a required string and returns the actual version if it doesn't match, or null if it does */
SAG_CONNECTIVITY_API const char * SAG_CONNECTIVITY_CALL ap_check_correlator_version(void*, const char *);

/** Marks this non-blocking method call as actually blocking */
SAG_CONNECTIVITY_API void SAG_CONNECTIVITY_CALL ap_plugin_method_blocking(void*);

/** @internal */
SAG_CONNECTIVITY_API void* SAG_CONNECTIVITY_CALL ap_get_correlator_internal(void*);
/** Fn pointer type for event handler functions */
typedef bool (*epl_call_eventhandler_fn_t)(void*, const char*, sag_underlying_data_t, const char*);
/** Fn pointer type for persistence handler functions */
typedef bool (*epl_call_persiststate_fn_t)(void*, void**, void**, uint64_t*, uint64_t*);
/** Fn pointer type for handlers to turn a chunk into a persistence ID */
typedef bool (*epl_call_getchunkid_fn_t)(void*, const sag_underlying_custom_t, uint64_t*);
/** Fn pointer type for handlers to create a custom_t from a void* */
/** Fn pointer type for row visitors */
typedef bool (*epl_visitrow_fn_t)(const void*, uint64_t, sag_underlying_vector_t);
/** Fn pointer type for flush handler functions */
typedef bool (*epl_call_flushhandler_fn_t)(void*, sag_underlying_map_t);
/** Fn pointer type for other handler functions */
typedef bool (*epl_call_handler_fn_t)(void*);
/** Fn pointer type for deleting handlers */
typedef void (*epl_delete_handler_fn_t)(void*);

/** Register an event handler for a plugin */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_register_plugin_eventhandler(void *, void *, const char *, uint64_t, epl_call_eventhandler_fn_t, epl_delete_handler_fn_t, bool);

/** Register a thread ended handler for a plugin */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_register_plugin_threadhandler(void *, void *, epl_call_handler_fn_t, epl_delete_handler_fn_t);

/** Add a channel to a plugin event handler */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_add_channel_eventhandler(void *, void *, const char*);
/** Remove a channel to a plugin event handler */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_remove_channel_eventhandler(void *, void *, const char*);

/** Return the current context ID */
SAG_CONNECTIVITY_API int64_t SAG_CONNECTIVITY_CALL ap_get_contextid(void *);

// persistence-related functions, internal

/** Register a flushAllQueues handler for a plugin */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_register_plugin_flushhandler(void *, void *, epl_call_flushhandler_fn_t, epl_delete_handler_fn_t);
/** Register persistent callbacks for a plugin */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_register_plugin_persistentcallbacks(void *, void *, epl_call_persiststate_fn_t, epl_call_getchunkid_fn_t, epl_delete_handler_fn_t);

/** Mark a chunk as dirty */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_chunk_dirty(void *, void *);
/** Mark a chunk as deleted */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_chunk_deleted(void*, uint64_t, void*);
/** Mark a chunk as created */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_chunk_created(void *, uint64_t, void *);
/** Mark a chunk as recovered */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_chunk_recovered(void *, uint64_t, sag_underlying_custom_t);
/** Get a unique ID for persistence */
SAG_CONNECTIVITY_API uint64_t SAG_CONNECTIVITY_CALL ap_persistence_getunique(void*);
/** Return true if the correlator is persistent */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_ispersistentcorrelator(void*);
/** Return true if the correlator is persistent and the monitor being called from is persistent */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_ispersistentmonitor(void *);
/** Open a table in the persistence DB */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_opentable(void*, const char *, sag_underlying_vector_t, sag_underlying_vector_t, void **);
/** Update a row in a table */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_updaterow(void*, void*, uint64_t, sag_underlying_vector_t);
/** Remove a row from a table */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_removerow(void*, void*, uint64_t);
/** Read all rows in a table */
SAG_CONNECTIVITY_API bool SAG_CONNECTIVITY_CALL ap_persistence_visitallrows(void*, void*, const void*, epl_visitrow_fn_t);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus


#endif // _SAG_INTERNAL_C_EPLPLUGIN_FUNCTIONS_HPP_

