/*
 * Title:        sag_internal/epl_plugin_macros.hpp
 * Description:  Implementation of macros for generating C ABI for EPL plugins
 * $Copyright (c) 2015-2016, 2022 Software AG, Darmstadt, Germany and/or Software AG USA Inc., Reston, VA, USA, and/or its subsidiaries and/or its affiliates and/or their licensors.$
 * Use, reproduction, transfer, publication or disclosure is prohibited except as specifically provided for in your License Agreement with Software AG
 */

#ifndef _SAG_INTERNAL_EPLPLUGIN_MACROS_HPP_
#define _SAG_INTERNAL_EPLPLUGIN_MACROS_HPP_


////////////////
// EPL plugins stuff

/** @internal */
#define _APAMA_DECLARE_EPL_PLUGIN(Class) \
	extern "C" { \
	SAG_CONNECTIVITY_DLL_SYM const char * SAG_CONNECTIVITY_CALL ap_epl_get_plugin_name() { return #Class; } \
	SAG_CONNECTIVITY_DLL_SYM uint64_t SAG_CONNECTIVITY_CALL ap_epl_api_version() { return SAG_PLUGIN_API_VERSION; } \
	SAG_CONNECTIVITY_DLL_SYM void SAG_CONNECTIVITY_CALL ap_epl_free_plugin_metadata_##Class(ApamaEPLMethodData *data) \
	{ \
		for (ApamaEPLMethodData *it = data; it->fn; ++it) { \
			free(it->signature); \
		} \
		delete[] data; \
	} \
	SAG_CONNECTIVITY_DLL_SYM ApamaEPLMethodData * SAG_CONNECTIVITY_CALL ap_epl_initialize_plugin_##Class(void *correlator) \
	{ \
		try { \
			com::apama::epl::MethodData<Class> mdata; \
			Class::setCorrelator(com::apama::epl::CorrelatorInterface(correlator)); \
			Class::initialize(mdata); \
			return mdata.getMethods(); \
		} catch (const std::exception &se) { \
			com::apama::epl::Logger(#Class).error("Failed to initialize plugin " #Class" : %s", se.what()); \
			return 0; \
		} catch (...) { \
			com::apama::epl::Logger(#Class).error("Failed to initialize plugin " #Class); \
			return 0; \
		} \
	}\
	SAG_CONNECTIVITY_DLL_SYM void * SAG_CONNECTIVITY_CALL ap_epl_create_plugin_##Class() \
	{ \
		try { \
			return new Class(); \
		} catch (const std::exception &se) { \
			com::apama::epl::Logger(#Class).error("Failed to create plugin instance " #Class" : %s", se.what()); \
			return 0; \
		} catch (...) { \
			com::apama::epl::Logger(#Class).error("Failed to create plugin instance " #Class); \
			return 0; \
		} \
	} \
	SAG_CONNECTIVITY_DLL_SYM void SAG_CONNECTIVITY_CALL ap_epl_destroy_plugin_##Class(void *object) \
	{ \
		try { \
			delete reinterpret_cast<Class*>(object); \
		} catch (...) { \
			com::apama::epl::Logger(#Class).error("Failed to destroy plugin instance " #Class); \
		} \
	} \
	SAG_CONNECTIVITY_DLL_SYM bool SAG_CONNECTIVITY_CALL ap_epl_call_function_##Class(void *object, void *fn, sag_underlying_vector_t args, sag_underlying_data_t *rv2) \
	{ \
		rv2->tag = SAG_DATA_EMPTY; \
		try { \
			com::apama::epl::data_t rv = Class::callMethod(object, fn, reinterpret_cast<com::apama::epl::list_t &>(args)); \
			rv.swap(std::move(reinterpret_cast<com::apama::epl::data_t&>(*rv2))); \
		} catch (const std::exception &e) { \
			com::apama::epl::data_t d(e.what());\
			d.swap(std::move(reinterpret_cast<com::apama::epl::data_t&>(*rv2))); \
			return false; \
		} catch (...) { \
			com::apama::epl::data_t d("Unknown exception");\
			d.swap(std::move(reinterpret_cast<com::apama::epl::data_t&>(*rv2))); \
			return false; \
		} \
		return true; \
	} \
}

#endif // _SAG_INTERNAL_EPLPLUGIN_MACROS_HPP_
